`timescale 1ns/1ps

module shallow_async_fifo_tb();

logic clk_a;
logic clk_b;

initial begin
    clk_a <= '1;
    forever begin
        #10 clk_a = ~clk_a;
    end
end

initial begin
    clk_b <= '1;
    forever begin
        #13 clk_b = ~clk_b;
    end
end

logic rst_a, rst_b, w_en, r_en, o_wrdy, o_rrdy;
logic [31:0] i_data, o_data;

shallow_async_fifo #(
    .WIDTH(32)
) u_shallow_async_fifo (
    .i_rst_a(rst_a),
    .i_clk_a(clk_a),
    .i_rst_b(rst_b),
    .i_clk_b(clk_b),
    .w_en(w_en),
    .i_data(i_data),
    .o_wrdy(o_wrdy),
    .r_en(r_en),
    .o_data(o_data),
    .o_rrdy(o_rrdy)
);

task write(input [31:0] data);
    $display("Writing to FIFO");
    while (~o_wrdy) begin
        $display("FIFO is full!");
        @(posedge clk_a);
    end
    i_data <= data;
    w_en <= '1;
    @(posedge clk_a);
    w_en <= '0;
    @(posedge clk_a);
endtask

task read(output [31:0] data);
    $display("Reading from FIFO");
    while (~o_rrdy) begin
        $display("FIFO is empty!");
        @(posedge clk_b);
    end
    r_en <= '1;
    data <= o_data;
    @(posedge clk_b);
    $display("Read data: %x\n", data);
    r_en <= '0;
    @(posedge clk_b);
endtask

logic [31:0] rd_data;

initial begin
    rst_a <= 1;
    rst_b <= 1;
    r_en <= 0;
    w_en <= 0;
    repeat (4) @(posedge clk_a);
    rst_a <= 0;
    @(posedge clk_a)
    rst_b <= 0;
    @(posedge clk_b)

    @(posedge clk_a)

    for (int i = 1; i < 5; i++) begin
        write(i);
    end

    for (int i = 1; i < 5; i++) begin
        write(i);
    end
end

initial begin
    repeat (5) @(posedge clk_a);
    @(posedge clk_b);
    @(posedge clk_a);

    for (int i = 1; i < 5; i++) begin
        read(rd_data);
    end

    for (int i = 1; i < 5; i++) begin
        read(rd_data);
    end

    $finish();
end

initial begin
    $dumpfile("shallow_async_fifo_tb.vcd");
    $dumpvars(0,shallow_async_fifo_tb);
    $dumpvars(0,shallow_async_fifo_tb.u_shallow_async_fifo.mem[0]);
    $dumpvars(0,shallow_async_fifo_tb.u_shallow_async_fifo.mem[1]);
end

endmodule