module apb4_decoder #(
    parameter AW = 32,
    parameter DW = 32,
    parameter NS = 1,
    parameter OFFSET_ADDRESSES = 1,
    parameter [NS*AW*2-1:0] SLAVE_ADDR = {
        {'0, '0}
    }
)(
    input   wire                    s_apb_psel,
    input   wire                    s_apb_penable,
    input   wire                    s_apb_pwrite,
    input   wire    [2:0]           s_apb_pprot,
    input   wire    [AW-1:0]        s_apb_paddr,
    input   wire    [DW-1:0]        s_apb_pwdata,
    input   wire    [DW/8-1:0]      s_apb_pstrb,
    output  logic                   s_apb_pready,
    output  logic   [DW-1:0]        s_apb_prdata,
    output  logic                   s_apb_pslverr,

    output  logic   [NS-1:0]        m_apb_psel,
    output  logic   [NS-1:0]        m_apb_penable,
    output  logic   [NS-1:0]        m_apb_pwrite,
    output  logic   [NS*3-1:0]      m_apb_pprot,
    output  logic   [NS*AW-1:0]     m_apb_paddr,
    output  logic   [NS*DW-1:0]     m_apb_pwdata,
    output  logic   [NS*DW/8-1:0]   m_apb_pstrb,
    input   wire    [NS-1:0]        m_apb_pready,
    input   wire    [NS*DW-1:0]     m_apb_prdata,
    input   wire    [NS-1:0]        m_apb_pslverr
);

localparam SEL_BITS = $clog2(NS);

logic [SEL_BITS:0] selected;
logic none_sel;


// Target Selection
always_comb begin
    selected = '0;
    none_sel = '1;

    for (int i = 0; i < NS; i++) begin
        if (
            s_apb_paddr >= SLAVE_ADDR[i*AW*2    +: AW] && 
            s_apb_paddr <= SLAVE_ADDR[i*AW*2+AW +: AW]
        ) begin
            selected = i;
            none_sel = '0;
        end
    end
end

always_comb begin

    m_apb_psel = '0;
    m_apb_penable = '0;
    m_apb_pwrite = '0;
    m_apb_pprot = '0;
    m_apb_paddr = '0;
    m_apb_pwdata = '0;
    m_apb_pstrb = '0;

    if (s_apb_psel && !none_sel) begin
        m_apb_psel[selected] = s_apb_psel;
        m_apb_penable[selected] = s_apb_penable;
        m_apb_pwrite[selected] = s_apb_pwrite;
        m_apb_pprot[selected*3 +: 3] = s_apb_pprot;

        if (OFFSET_ADDRESSES) begin
            m_apb_paddr[selected*AW +: AW] = s_apb_paddr - SLAVE_ADDR[selected*AW*2 +: AW];
        end else begin
            m_apb_paddr[selected*AW +: AW] = s_apb_paddr;
        end

        m_apb_pwdata[selected*DW +: DW] = s_apb_pwdata;
        m_apb_pstrb[selected*(DW/8) +: (DW/8)] = s_apb_pstrb;
        s_apb_pready = m_apb_pready[selected];
        s_apb_prdata = m_apb_prdata[selected*DW +: DW];
        s_apb_pslverr = m_apb_pslverr[selected];
    end else begin
        s_apb_pready = '0;
        s_apb_prdata = '0;
        s_apb_pslverr = '0;
    end
end

endmodule