// simple axi rom . Only supports 32 bit aligned reads.

module axi4_lite_rom #(
    parameter ROM_INIT_FILE = "init_hex.mem",

    parameter BASE_ADDRESS = 32'h00000000,

    parameter AXI_ADDR_WIDTH = 32,
    parameter AXI_DATA_WIDTH = 32,

    // This is byte address width!
    parameter ROM_SIZE = 16
)(
    input i_clk,
    input i_rst,

    input i_AWVALID,
    output logic o_AWREADY,
    input [AXI_ADDR_WIDTH-1:0] i_AWADDR,
    input [2:0] i_AWPROT,

    input i_WVALID,
    output logic o_WREADY,
    input [AXI_DATA_WIDTH-1:0] i_WDATA,
    input [AXI_DATA_WIDTH/8-1:0] i_WSTRB,

    output logic o_BVALID,
    input i_BREADY,
    output logic [1:0] o_BRESP,

    input i_ARVALID,
    output logic o_ARREADY,
    input [AXI_ADDR_WIDTH-1:0] i_ARADDR,
    input [2:0] i_ARPROT,

    output logic o_RVALID,
    input i_RREADY,
    output logic [AXI_DATA_WIDTH-1:0] o_RDATA,
    output logic [1:0] o_RRESP
);

localparam MEMORY_DEPTH = 1 << ROM_SIZE;

logic [AXI_DATA_WIDTH-1:0] rom [MEMORY_DEPTH];


initial begin
    $readmemh(ROM_INIT_FILE, rom);
end

logic state;

logic [AXI_ADDR_WIDTH-3:0] addr;

logic [AXI_ADDR_WIDTH-1:0] local_addr;

assign local_addr = i_ARADDR - BASE_ADDRESS;


always @(posedge i_clk) begin
    if (i_rst) begin
        o_RVALID <= '0;
    end else begin
        if (i_ARVALID) begin
            o_RDATA <= rom[local_addr[AXI_ADDR_WIDTH-1:2]];
            o_RVALID <= '1;
        end else begin
            o_RVALID <= '0;
        end
    end
end


assign o_WREADY = '0;
assign o_AWREADY = '0;
assign o_BVALID = '0;
assign o_ARREADY = '1;
assign o_RRESP = '0;


endmodule