// Copyright (c) 2023 Byron Lathi

module sync_fifo #(
    parameter DEPTH=8,
    parameter WIDTH=8
)(
    input i_rst,
    input i_clk,
    input w_en,
    input [WIDTH-1:0] i_data,
    output o_full,

    input r_en,
    output [WIDTH-1:0] o_data,
    output o_empty
);

localparam AW = $clog2(DEPTH);

logic [AW:0] w_ptr, w_ptr_next;
logic [AW:0] r_ptr, r_ptr_next;

logic full, full_next;
logic empty, empty_next;

logic [WIDTH-1:0] mem [DEPTH];

logic [WIDTH-1:0] mem_out;

always @(posedge i_clk or posedge i_rst) begin
    if (i_rst) begin
        w_ptr <= '0;
        r_ptr <= '0;
        full <= '0;
        empty <= '1;
    end else begin
        w_ptr <= w_ptr_next;
        r_ptr <= r_ptr_next;
        empty <= empty_next;
        full <= full_next;
    end

    if (w_en && !full) begin
        mem[w_ptr[AW-1:0]] <= i_data;
    end
end

always_comb begin
    w_ptr_next = w_ptr + (w_en && !full);
    r_ptr_next = r_ptr + (r_en && !empty);

    full_next = w_ptr_next == {~r_ptr_next[AW], r_ptr_next[AW-1:0]};
    empty_next = r_ptr_next == w_ptr_next;

    mem_out = mem[r_ptr[AW-1:0]];
end

assign o_full = full;
assign o_empty = empty;
assign o_data = mem_out;

endmodule