// Copyright (c) 2024 Byron Lathi

module sync_fwft_fifo #(
    parameter DEPTH,
    parameter WIDTH
)(
    input i_rst,
    input i_clk,
    input w_en,
    input [WIDTH-1:0] i_data,
    output o_full,

    input r_en,
    output [WIDTH-1:0] o_data,
    output o_empty
);


    logic                   dout_valid;
    logic                   fifo_rd_en, fifo_empty;

    sync_fifo #(
        .DEPTH(DEPTH),
        .WIDTH(WIDTH)
    ) u_fifo (
        .i_rst      (i_rst),
        .i_clk      (i_clk),
        .w_en       (w_en),
        .i_data     (i_data),
        .o_full     (o_full),
        .r_en       (r_en),
        .o_data     (o_data),
        .o_empty    (fifo_empty)
    );

    assign fifo_rd_en = !fifo_empty && (!dout_valid || r_en);
    assign o_empty = !dout_valid;

    always_ff@(posedge i_clk or posedge i_rst) begin
        if (i_rst) begin
            dout_valid <= 0;
        end else begin
            if (fifo_rd_en) begin
                dout_valid <= 1;
            end else if (r_en) begin
                dout_valid <= 0;
            end
        end
    end
endmodule